@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt_io.h 20179 2018-05-27 03:00:30Z pwessel $
  *
- *	Copyright (c) 1991-2018 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2020 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  * Table input/output in GMT can be either ASCII or binary (where supported)
@@ -24,32 +23,33 @@
  *
  * Author:	Paul Wessel
  * Date:	15-NOV-2009
- * Version:	5 API
+ * Version:	6 API
  *
  */
 
 /*!
  * \file gmt_io.h
- * \brief  
+ * \brief
  */
 
-#ifndef _GMT_IO_H
-#define _GMT_IO_H
+#ifndef GMT_IO_H
+#define GMT_IO_H
 
 #ifdef HAVE_SETLOCALE
 #	include <locale.h>
 #endif
 
-static inline const char* __gmt_token_separators (void) {
+static inline const char* __gmt_token_separators (unsigned int skip_comma) {
 	static const char separators[] = ",; \t";
 #ifdef HAVE_SETLOCALE
 	struct lconv *lc = localeconv();
-	if ( (strcmp (lc->decimal_point, ",") == 0) )
+	if (skip_comma || (strcmp (lc->decimal_point, ",") == 0) )
 		return separators + 1; /* Omit comma */
 #endif
 	return separators;
 }
-#define GMT_TOKEN_SEPARATORS __gmt_token_separators() /* Data columns may be separated by any of these characters */
+#define GMT_TOKEN_SEPARATORS __gmt_token_separators(0) /* Data columns may be separated by any of these characters */
+#define GMT_TOKEN_SEPARATORS_PSTEXT __gmt_token_separators(1) /* No comma if pstext and fonts are in input records */
 
 /* Must add M, m, E, Z, and/or S to the common option processing list */
 #define GMT_OPT(opt) opt
@@ -68,17 +68,6 @@ enum GMT_ogr_status {
 
 #define gmt_M_polygon_is_hole(S) (S->pol_mode == GMT_IS_HOLE || (S->ogr && S->ogr->pol_mode == GMT_IS_HOLE))
 
-/* Specific feature geometries as obtained from OGR */
-/* Note: As far as registering or reading data, GMT only needs to know if data type is POINT, LINE, or POLY */
-
-enum GMT_enum_ogr {
-	GMT_IS_LINESTRING	= 2U,
-	GMT_IS_POLYGON		= 4U,
-	GMT_IS_MULTI		= 64U,	/* Flag for multi-versions of geometry */
-	GMT_IS_MULTIPOINT	= 65U,	/* 1 plus 64 for multi */
-	GMT_IS_MULTILINESTRING	= 66U,	/* 2 plus 64 for multi */
-	GMT_IS_MULTIPOLYGON	= 68U};	/* 4 plus 64 for multi */
-
 /*! Codes for aspatial association with segment header options: */
 enum GMT_enum_segopt {
 	/*! -D */	GMT_IS_D = -1,
@@ -105,7 +94,7 @@ enum GMT_enum_segopt {
 /* Get current setting for in/out columns */
 
 /*! Types of possible column entries in a file: */
-enum gmt_M_col_enum {
+enum gmt_col_enum {
 	GMT_IS_NAN   		=   0,	/* Returned by gmt_scanf routines when read fails */
 	GMT_IS_FLOAT		=   1,	/* Generic (double) data type, no special format */
 	GMT_IS_LAT		=    2,
@@ -132,7 +121,8 @@ enum GMT_lon_enum {
 	GMT_IS_M360_TO_0			= 4,	/* Report -360 < lon <= 0 */
 	GMT_IS_M180_TO_P180_RANGE	= 5,	/* Report -180 <= lon <= +180 */
 	GMT_IS_M180_TO_P180			= 6,	/* Report -180 <= lon < +180 */
-	GMT_IS_M180_TO_P270_RANGE	= 7};	/* Report -180 <= lon < +270 [GSHHG only] */
+	GMT_IS_M180_TO_P270_RANGE	= 7,	/* Report -180 <= lon < +270 [GSHHG only] */
+	GMT_IGNORE_RANGE	= 99};		/* Do not adjust longitudes at all */
 
 /*! How to handle NaNs in records */
 enum GMT_io_nan_enum {
@@ -184,7 +174,7 @@ struct GMT_CLOCK_IO {
 	unsigned int n_sec_decimals;	/* Number of digits in decimal seconds (0 for whole seconds) */
 	bool compact;		/* true if we do not want leading zeros in items (e.g., 03) */
 	bool twelve_hr_clock;	/* true if we are doing am/pm on output */
-	char ampm_suffix[2][8];		/* Holds the strings to append am or pm */
+	char ampm_suffix[2][GMT_LEN8];	/* Holds the strings to append am or pm */
 	char format[GMT_LEN64];	/* Actual C format used to output clock */
 	char delimiter[2][2];		/* Delimiter strings in clock, e.g. ":" */
 };
@@ -222,7 +212,7 @@ struct GMT_COL_INFO {	/* Used by -i and input parsing */
 	unsigned int order;	/* The initial order (0,1,...) but this will be sorted on col */
 	unsigned int convert;	/* 2 if we must convert the data by log10, 1 if scale, offset */
 	double scale;		/* Multiplier for raw in value */
-	double offset;		/* Offset applied after multiplier */ 
+	double offset;		/* Offset applied after multiplier */
 };
 
 struct GMT_COL_TYPE {	/* Used by -b for binary formatting */
@@ -231,12 +221,23 @@ struct GMT_COL_TYPE {	/* Used by -b for binary formatting */
 	int (*io) (struct GMT_CTRL *, FILE *, uint64_t, double *);	/* Pointer to the correct read or write function given type/swab */
 };
 
+/*! For selecting row ranges via -q */
+struct GMT_ROW_RANGE {
+	int64_t first, last, inc;
+};
+
+/*! For selecting data ranges via -q */
+struct GMT_DATA_RANGE {
+	double first, last;
+};
+
 struct GMT_IO {				/* Used to process input data records */
 	void * (*input) (struct GMT_CTRL *, FILE *, uint64_t *, int *);	/* Pointer to function reading ASCII or binary tables */
-	int (*output) (struct GMT_CTRL *, FILE *, uint64_t, double *);	/* Pointer to function writing ASCII or binary tables */
+	int (*output) (struct GMT_CTRL *, FILE *, uint64_t, double *, char *);	/* Pointer to function writing ASCII or binary tables */
 	int (*read_item) (struct GMT_CTRL *, FILE *, uint64_t, double *);		/* Pointer to function reading 1-col z tables in grd2xyz */
 	int (*write_item) (struct GMT_CTRL *, FILE *, uint64_t, double *);		/* Pointer to function writing 1-col z tables in xyz2grd */
 	bool (*ogr_parser) (struct GMT_CTRL *, char *);				/* Set to handle either header or data OGR records */
+	const char *scan_separators;	/* List of characters that separates columns in ascii records */
 
 	unsigned int pad[4];		/* pad[0] = west, pad[1] = east, pad[2] = south, pad[3] = north */
 	unsigned int inc_code[2];
@@ -248,16 +249,26 @@ struct GMT_IO {				/* Used to process input data records */
 	bool skip_bad_records;	/* true if records where x and/or y are NaN or Inf */
 	bool give_report;		/* true if functions should report how many bad records were skipped */
 	bool skip_duplicates;	/* true if we should ignore duplicate x,y records */
-	bool read_mixed;		/* true if we are reading ASCII x y [z] [variable numbers of text] */
+	bool variable_in_columns;	/* true if we are reading ASCII records with variable numbers of columns */
 	bool need_previous;		/* true if when parsing a record we need access to previous record values (e.g., for gap or duplicate checking) */
+	bool has_previous_rec;		/* true if we have the previous record for this segment */
 	bool warn_geo_as_cartesion;	/* true if we should warn if we read a record with geographic data while the expected format has not been set (i.e., no -J or -fg) */
-
+	bool first_rec;			/* true when reading very first data record in a dataset */
+	bool trailing_text[2];	/* Default is to process training text unless turned off via -i, -o */
+	bool refreshed[2];		/* true after calling the hash_refresh function the first time, for hash and info, respectively */
+	bool internet_error;		/* true after failing to get hash table due to time-out */
+	bool grid_padding;		/* If true we try to read two extra rows/cols from grids for BC purposes */
+	bool leave_as_jp2;		/* If true we do not convert downloaded JP2 grids to NC right away, but as needed */
 	uint64_t seg_no;		/* Number of current multi-segment in entire data set */
 	uint64_t seg_in_tbl_no;		/* Number of current multi-segment in current table */
 	uint64_t n_clean_rec;		/* Number of clean records read (not including skipped records or comments or blanks) */
 	uint64_t n_bad_records;		/* Number of bad records encountered during i/o */
 	unsigned int tbl_no;		/* Number of current table in entire data set */
 	unsigned int io_nan_ncols;	/* Number of columns to consider for -s option */
+	unsigned int record_type[2];	/* Either GMT_READ|WRITE_DATA (0), GMT_READ|WRITE_TEXT (1), or GMT_READ|WRITE_MIXED (2) (for input and output) */
+	unsigned int n_numerical_cols;	/* As it says */
+	unsigned int max_cols_to_read;	/* For ascii input [all] */
+	unsigned int n_row_ranges[2];	/* How many row ranges given in -q */
 	enum GMT_ogr_status ogr;	/* Tells us if current input source has OGR/GMT metadata (GMT_OGR_TRUE) or not (GMT_OGR_FALSE) or not set (GMT_OGR_UNKNOWN) */
 	unsigned int status;		/* 0	All is ok
 					   1	Current record is segment header
@@ -266,17 +277,24 @@ struct GMT_IO {				/* Used to process input data records */
 					   8	NaNs encountered in first 2/3 cols */
 	uint64_t rec_no;		/* Number of current records (counts headers etc) in entire data set */
 	uint64_t rec_in_tbl_no;		/* Number of current record (counts headers etc) in current table */
-	uint64_t pt_no;			/* Number of current valid points in a row  */
+	uint64_t data_record_number_in_set[2];	/* Number of current valid data record number in the whole dataset, for input and output. Headers not counted.  */
+	uint64_t data_record_number_in_tbl[2];	/* Number of current valid data record number in the current table, for input and output. Headers not counted.  */
+	uint64_t data_record_number_in_seg[2];	/* Number of current valid data record number in the current segment, for input and output. Headers not counted.  */
 	int64_t curr_pos[2][4];		/* Keep track of current input/output table, segment, row, and table headers (for rec-by-rec action) */
 	char r_mode[4];			/* Current file opening mode for reading (r or rb) */
 	char w_mode[4];			/* Current file opening mode for writing (w or wb) */
 	char a_mode[4];			/* Current file append mode for writing (a+ or ab+) */
-	char record[GMT_BUFSIZ];	/* Current ASCII record */
+	char curr_text[GMT_BUFSIZ];	/* Current ASCII record as it was read */
+	char curr_trailing_text[GMT_BUFSIZ];	/* Current text portion of current record (or NULL) */
 	char segment_header[GMT_BUFSIZ];	/* Current ASCII segment header */
-	char filename[2][GMT_BUFSIZ];	/* Current filenames (or <stdin>/<stdout>) */
+	char filename[2][PATH_MAX];	/* Current filenames (or <stdin>/<stdout>) */
+#ifdef HAVE_GDAL
+	char tempfile[PATH_MAX];	/* Temporary file used to read - should be removed when closed */
+#endif
+	char col_set[2][GMT_MAX_COLUMNS];	/* Keeps track of which columns have had their type set */
 	char *o_format[GMT_MAX_COLUMNS];	/* Custom output ASCII format to overrule format_float_out */
 	int ncid;			/* NetCDF file ID (when opening netCDF file) */
-	int nvars;			/* Number of requested variablesin netCDF file */
+	int nvars;			/* Number of requested variables in netCDF file */
 	uint64_t ncols;			/* Number of total columns in netCDF file */
 	size_t t_index[GMT_MAX_COLUMNS][5];		/* Indices for cross-sections (netCDF only) */
 	size_t count[GMT_MAX_COLUMNS][5];		/* Count used for cross-sections (netCDF only) */
@@ -293,7 +311,11 @@ struct GMT_IO {				/* Used to process input data records */
 	unsigned int io_nan_col[GMT_MAX_COLUMNS];	/* Array of columns to consider for -s option ir true */
 	struct GMT_COL_INFO col[2][GMT_MAX_COLUMNS];	/* Order of columns on input and output unless 0,1,2,3,... */
 	struct GMT_COL_TYPE fmt[2][GMT_MAX_COLUMNS];	/* Formatting information for binary data */
+	struct GMT_ROW_RANGE row_range[2][GMT_MAX_RANGES];		/* One or more ranges for input or output rows */
+	struct GMT_DATA_RANGE data_range[2][GMT_MAX_RANGES];		/* One or more ranges for input or output times */
 	struct GMT_OGR *OGR;		/* Pointer to GMT/OGR info used during reading */
+	struct GMT_RECORD record;	/* Current record with pointers to data columns and text */
+	double *nc_xarray, *nc_yarray;	/* For grids with variable x,y arrays */
 	/* The remainder are just pointers to memory allocated elsewhere */
 	int *varid;			/* Array of variable IDs (netCDF only) */
 	double *scale_factor;		/* Array of scale factors (netCDF only) */
@@ -339,4 +361,4 @@ struct GMT_PLOT_CALCLOCK {
 
 /* For the GMT_GRID container, see gmt_grdio.h */
 
-#endif /* _GMT_IO_H */
+#endif /* GMT_IO_H */
